#!/usr/bin/perl

#W_RELEASE ... ARGs: force_constant eq_value temp
#calculates numerical (simulation) release work for orientational restraint.  Calcs analyitical for positional restraint.

$k_r=5.0; # kcal/mol-A^2
$r1_0=21.00; #$r0=15.075;    # equilibrium distance
$temp=$ARGV[0];
$k_a=100; # kcal/mol-rad^2
$pi=3.1415926535897932384626433832795;
$pin=-3.1415926535897932384626433832795;
$k_b=1.987204118e-3; # kcal/mol-K  Also R
$beta=1/($temp*$k_b);
#print "beta= $beta\n";

($r1lb,$r1ub,$r1st)=(0,100,0.0001);
($a1lb,$a1ub,$a1st)=(0,$pi,0.00005);
($t1lb,$t1ub,$t1st)=(-$pi,$pi,0.00003);
($a2lb,$a2ub,$a2st)=(0,$pi,0.00003);
($t2lb,$t2ub,$t2st)=(-$pi,$pi,0.00003);
($t3lb,$t3ub,$t3st)=(-$pi,$pi,0.00003);


sub d2r  {return ($_[0]*$pi/180);}
$a1_0=d2r(90.00);  # equilibrium positional angle
$t1_0=d2r(0.00);  # equilibrium positional dihedral
$a2_0=d2r(aa22);  # equilibrium orientational angle
$t2_0=d2r(tt22);  # equilibrium orientational dihedral 1
$t3_0=d2r(tt33);  # equilibrium orientational dihedral 2
sub f_r1 {return ( ($_[0]**2)*exp(-$beta*$k_r*($_[0]-$r1_0)**2) );}
sub f_a1 {return ( sin($_[0])*exp(-$beta*$k_a*($_[0]-$a1_0)**2) );}
sub f_a2  {return ( sin($_[0])*exp(-$beta*$k_a*($_[0]-$a2_0)**2) );}
sub f_t1 {
$delta=($_[0]-$t1_0);
if ($delta > $pi) {
$delta=$delta-(2*$pi);
}
return ( exp(-$beta*$k_a*($delta)**2) );
}
sub f_t2 {
$delta=($_[0]-$t2_0);
if ($delta > $pi) {
$delta=$delta-(2*$pi);
}
if ($delta < $pin) {
$delta=$delta+(2*$pi);
}
return ( exp(-$beta*$k_a*($delta)**2) );
}
sub f_t3 {
$delta=($_[0]-$t3_0);
if ($delta > $pi) {
$delta=$delta-(2*$pi);
}
if ($delta < $pin) {
$delta=$delta+(2*$pi);
}
return ( exp(-$beta*$k_a*($delta)**2) );
}

# Integrate r1  (positional)
for ($i=$r1lb;$i<=$r1ub;$i+=$r1st){
  $r1int+=($r1st)*( (f_r1($i-$r1st)+f_r1($i))/2 );
}
  
# Integrate a1 (positional)
for ($i=$a1lb;$i<=$a1ub;$i+=$a1st){
  $a1int+=($a1st)*( (f_a1($i-$a1st)+f_a1($i))/2 );
}

# Integrate t1 (positional)
for ($i=$t1lb;$i<=$t1ub;$i+=$t1st){
  $t1int+=($t1st)*( (f_t1($i-$t1st)+f_t1($i))/2 );
}

# Integrate a2 (orientational)
for ($i=$a2lb;$i<=$a2ub;$i+=$a2st){
  $a2int+=($a2st)*( (f_a2($i-$a2st)+f_a2($i))/2 );
}

# Integrate t2 (orientational)
for ($i=$t2lb;$i<=$t2ub;$i+=$t2st){
  $t2int+=($t2st)*( (f_t2($i-$t2st)+f_t2($i))/2 );
}

# Integrate t3 (orientational)
for ($i=$t3lb;$i<=$t3ub;$i+=$t3st){
  $t3int+=($t3st)*( (f_t3($i-$t3st)+f_t3($i))/2 );
}


$tot_p=$r1int*$a1int*$t1int;

$tot_o=$a2int*$t2int*$t3int;

#print "\n\n$rint\n$tint\n$oint\n$tot_p\n\n";


## This is the Work to release molecule from standard conc.  thus +K_b
$W_r1=$k_b*$temp*log((1/(1660))*$r1int);
$W_a1=$k_b*$temp*log((1/(1660))*$a1int);
$W_t1=$k_b*$temp*log((1/(1660))*$t1int);
$W_t=$k_b*$temp*log((1/(1660))*$tot_p);

## This is the Work to release orientation to freely rotating.
$W_a2=$k_b*$temp*log((1/(8*($pi**2)))*$a2int);
$W_t2=$k_b*$temp*log((1/(8*($pi**2)))*$t2int);
$W_t3=$k_b*$temp*log((1/(8*($pi**2)))*$t3int);
$W_o=$k_b*$temp*log((1/(8*($pi**2)))*$tot_o);

#printf "\nW_orient: %.4f\n",$W_o; 
#print "\nW_r: $W_r\tW_thet: $W_t\nW_r+thet: $W  (at $ARGV[0] )\n";
#printf "\nW_TOTAL: %8.3f\n", $W_o+$W;
#print "$W\n";
printf "%12.7f\n", $r1int;
printf "%12.7f\n", $a1int;
printf "%12.7f\n", $t1int;
printf "%12.7f\n", $a2int;
printf "%12.7f\n", $t2int;
printf "%12.7f\n", $t3int;
printf "%12.7f\n", $W_t;
printf "%12.7f\n", $W_o;
printf "%12.7f\n", $W_t+$W_o;

