#!/bin/bash

##############################################################################################################
########################## Calcuation of the binding free energy using MBAR###################################
##############################################################################################################
#
# Instructions: 
#
# 1 - Copy this script to the main folder of the chosen molecule calculations Ex: ./BZT-7-SPC
#
# 2 - Run the script on this folder as $./get-results.bash <mol> , where mol is the molecule ID in lower case 
# (Ex: 08k). Make sure you have the necessary programs installed and in your path (pymbar, akima, amber, etc.)
#
# 3 - The complete results are written to the Results.txt file
#

mol=$1

cp -r ../Base-SPC/analysis/ ./
cp ./analysis/TR-orig.pl ./
cd BRD4-$mol/u00
a2=$(cat disang.rest | awk 'NR==11{print $7}')
t2=$(cat disang.rest | awk 'NR==12{print $7}')
t3=$(cat disang.rest | awk 'NR==13{print $7}')
cd ../../
cp TR-orig.pl TR-release.pl
chmod +x TR-release.pl
sed -i "s/aa22/$a2/g" ./TR-release.pl
sed -i "s/tt22/$t2/g" ./TR-release.pl
sed -i "s/tt33/$t3/g" ./TR-release.pl
./TR-release.pl 298 > TR-Results.dat
nrb=$(cat TR-Results.dat | awk 'NR==9{print $1}')
rb=$(echo "(-1.00000)*${nrb}" | bc -l)
pr=1.73
npr=$(echo "(-1.00000)*${pr}" | bc -l)
npe=0.03

cd BRD4-$mol
cp -r ../../Base-SPC/fe/ ./
cp ../../Base-SPC/fe/output* ./
export PYTHONPATH="/home/microway/python-stuff/akima:/home/microway/python-stuff/pymbar-2.0beta/pymbar"
./output.bash
./output-la.bash
cd fe
python FE-MBAR-A.py a 298
python FE-MBAR-L.py l 298
python FE-MBAR.py u 298
cp mbar-a.100.dat mbar-a-eff.dat
cp mbar-l.100.dat mbar-l-eff.dat
cp mbar-u.100.dat mbar-u-eff.dat
cp mbar-a.log mbar-a-eff.log
cp mbar-l.log mbar-l-eff.log
cp mbar-u.log mbar-u-eff.log
python FE-MBAR-An.py a 298
python FE-MBAR-Ln.py l 298
python FE-MBAR-n.py u 298
cp mbar-a.100.dat mbar-a-tot.dat
cp mbar-l.100.dat mbar-l-tot.dat
cp mbar-u.100.dat mbar-u-tot.dat
cp mbar-a.log mbar-a-tot.log
cp mbar-l.log mbar-l-tot.log
cp mbar-u.log mbar-u-tot.log
a=$(cat mbar-a-tot.dat | awk 'NR==15{print $2}')
u=$(cat mbar-u-tot.dat | awk 'NR==41{print $2}')
l=$(cat mbar-l-tot.dat | awk 'NR==15{print $2}')
ae=$(cat mbar-a-eff.dat | awk 'NR==15{print $3}')
ue=$(cat mbar-u-eff.dat | awk 'NR==41{print $3}')
le=$(cat mbar-l-eff.dat | awk 'NR==15{print $3}')
cd ../../zzzz-$mol
if [ ! -d "fe" ]; then
cp -r ../../Base-SPC/z-fe/ ./fe
fi
cp ../../Base-SPC/z-fe/output* ./
./output.bash
cd fe
python FE-MBAR.py c 298
cp mbar-c.100.dat mbar-c-eff.dat
cp mbar-c.log mbar-c-eff.log
python FE-MBAR-n.py c 298
cp mbar-c.100.dat mbar-c-tot.dat
cp mbar-c.log mbar-c-tot.log
c=$(cat mbar-c-tot.dat | awk 'NR==15{print $2}')
nc=$(echo "(-1.00000)*${c}" | bc -l)
ce=$(cat mbar-c-eff.dat | awk 'NR==15{print $3}')
tot=$(echo "${a}+${u}+${l}+${nc}+${nrb}+${npr}" |bc -l)
tote2=$(echo "(${ae}*${ae})+(${le}*${le})+(${ue}*${ue})+(${ce}*${ce})+(${npe}*${npe})" |bc -l)
tote=$(echo "sqrt ( $tote2 )" | bc -l )
cd ../../
echo "Components" > Results.txt
echo "" >> Results.txt
printf "$a \t $ae \t  Attach protein\n" >> Results.txt
printf "$l \t $le \t  Attach ligand \n" >> Results.txt
printf "$u \t $ue \t  Ligand Pulling\n" >> Results.txt
printf "%0.5f \t 0.00000 \t  Release ligand TR\n" "$nrb" >> Results.txt
printf "%0.5f \t %0.5f \t  Release ligand CF\n" "$nc" "$ce"  >> Results.txt
printf "%0.5f \t %0.5f \t  Release protein\n" "$npr" "$npe" >> Results.txt
echo "" >> Results.txt
printf "DeltaG  \t  Uncertainty\n" >> Results.txt
echo "" >> Results.txt
printf "%0.5f \t  %0.5f" "$tot" "$tote" >> Results.txt
