#!/usr/bin/perl

if ($#ARGV < 0){die "Args: c|a|u|l RotAngle Weight% ScaleW H1 H2 H3 G1 G2 G3  \n\n";}

# Arguments
$Type=$ARGV[0];
$RotAngle=$ARGV[1];
$Weight=$ARGV[2];
$ScaleW=$ARGV[3];
$H1=$ARGV[4];
$H2=$ARGV[5];
$H3=$ARGV[6];
$G1=$ARGV[7];
$G2=$ARGV[8];
$G3=$ARGV[9];

# Fixed Values
$dkcal=5.0;
$akcal=100.0;
$jkcal=5.0;
$lkcal=5.0;
$hkcal=20.0;
$wkcal=500.0;
$ckcal=100.0;

# Host/Guest Translational/Rotational Restraints
$rst[0]=":2\@Pb $H1";
$rst[1]=":1\@Pb :2\@Pb $H1";
$rst[2]=":3\@Pb :1\@Pb :2\@Pb $H1";
$rst[3]=":2\@Pb $H1 $H2";
$rst[4]=":1\@Pb :2\@Pb $H1 $H2";
$rst[5]=":2\@Pb $H1 $H2 $H3";

$rst[6]=":1\@Pb $G1";
$rst[7]=":2\@Pb :1\@Pb $G1";
$rst[8]=":3\@Pb :2\@Pb :1\@Pb $G1";
$rst[9]=":1\@Pb $G1 $G2";
$rst[10]=":2\@Pb :1\@Pb $G1 $G2";
$rst[11]=":1\@Pb $G1 $G2 $G3";

# Guest Conformational Restraints
$rst[13]="$G1 $G2";
$rst[14]="$G2 $G3";
$rst[15]="$G3 $G1";

# Host Conformational Restraints
$rst[18]="$H1 $H2";
$rst[19]="$H2 $H3";
$rst[20]="$H3 $H1";

if ($Type eq "c"){
#Host Dihedral
$rst[17]=':2000@O  :2000@O  :2000@O  :2000@O';

#Guest Dihedral
$rst[12]=':4@DD1 :4@DD2 :4@DD3 :4@DD4';
$rst[16]=':4@DD1 :4@DD2 :4@DD3 :4@DD4';
}
else {
#Host Dihedral
$rst[17]=':48@N  :48@CA  :48@C  :49@N';

#Guest Dihedral
$rst[12]=':125@DD1 :125@DD2 :125@DD3 :125@DD4';
$rst[16]=':125@DD1 :125@DD2 :125@DD3 :125@DD4';
}

# Final Restraint Weight     | (kcal/mol-Ang^2  or kcal/mol-Rad^2)
#                            V
if ($Type eq "a"){
  $dkcal=0.0;
  $akcal=0.0;
  $dwt=sprintf "%.7f",0;
  $awt=sprintf "%.7f",0;
  $cwt=sprintf "%.7f",0;
  $lwt=sprintf "%.7f",0;
  $jwt=sprintf "%.7f",$Weight*$jkcal/100;
  $hwt=sprintf "%.7f",$Weight*$hkcal/100;
}
elsif ($Type eq "c"){
  $dkcal=0.0;
  $akcal=0.0;
  $dwt=sprintf "%.7f",0;
  $awt=sprintf "%.7f",0;
  $cwt=sprintf "%.7f",$Weight*$ckcal/100;
  $lwt=sprintf "%.7f",$Weight*$jkcal/100;
  $jwt=sprintf "%.7f",0;
  $hwt=sprintf "%.7f",0;
}
elsif ($Type eq "l"){
  $dwt=sprintf "%.7f",$Weight*$dkcal/100;
  $awt=sprintf "%.7f",$Weight*$akcal/100;
  $lwt=sprintf "%.7f",$Weight*$lkcal/100;
  $cwt=sprintf "%.7f",$Weight*$ckcal/100;
  $jwt=sprintf "%.7f",$jkcal;
  $hwt=sprintf "%.7f",$hkcal;
}
elsif ($Type eq "u"){
  $dwt=sprintf "%.7f",$Weight*$dkcal/100;
  $awt=sprintf "%.7f",$Weight*$akcal/100;
  $jwt=sprintf "%.7f",$Weight*$jkcal/100;
  $lwt=sprintf "%.7f",$Weight*$lkcal/100;
  $cwt=sprintf "%.7f",$Weight*$ckcal/100;
  $hwt=sprintf "%.7f",$Weight*$hkcal/100;
}


# Assign atom numbers for masks
@atoms=`grep 'ATOM ' vac.pdb`;
foreach (@atoms){
  @cols=split " ", $_;
  $atmnum{":$cols[4]\@$cols[2]"}=$cols[1];
}

# Write pt-assign.in and get reference restraint values
open OUT, ">pt-assign.in";
print OUT "#AnchorAtoms $H1 $H2 $H3 $G1 $G2 $G3\nparm full.prmtop\ntrajin full.crds\n";
for $i (0..$#rst){
#  if ($rst[$i] =~ /XXXX/){next;}
  @arr=split " ", $rst[$i];
  if ($#arr == 1){print OUT "distance r$i $rst[$i] noimage out assign.dat\n";}
  if ($#arr == 2){print OUT "angle r$i $rst[$i] out assign.dat\n";}
  if ($#arr == 3){print OUT "dihedral r$i $rst[$i] out assign.dat\n";}
}
close OUT;
system "cpptraj -i pt-assign.in >& pt-assign.log";
if ($Type eq "c") {
system "cp ./assign.dat ./assign-original.dat; cp ../../BRD4-mmm/u00/assign.dat ./"
}
@vals=split " ", `tail -n 1 assign.dat`;
shift(@vals);

# Write disang.rest. AMBER readable restraints
open OUT, ">disang.rest";
print OUT "#AnchorAtoms $H1 $H2 $H3 $G1 $G2 $G3 #Type $Type #Weight $Weight #RotAngle $RotAngle #ScaleW $ScaleW\n";
for $i (0..$#rst){
  if ($rst[$i] =~ /:5000\@O/ || $rst[$i] =~ /:2000\@O/){next;}
  @arr=split " ", $rst[$i];
  $atmnums="";
  foreach (@arr){
    $atmnums=$atmnums."$atmnum{$_},";
  }
  #HostTR
  if ($#arr == 1 && $i <= 5){printf OUT "&rst iat= %-23s r1= %10.4f, r2= %10.4f, r3= %10.4f, r4= %10.4f, rk2= %11.7f, rk3= %11.7f, &end #HostTR\n",$atmnums,0.0,$vals[$i],$vals[$i],999.0,$dkcal,$dkcal;}
  elsif ($#arr == 2 && $i <= 5){printf OUT "&rst iat= %-23s r1= %10.4f, r2= %10.4f, r3= %10.4f, r4= %10.4f, rk2= %11.7f, rk3= %11.7f, &end #HostTR\n",$atmnums,0.0,$vals[$i],$vals[$i],180.0,$akcal,$akcal;}
  elsif ($#arr == 3 && $i <= 5){printf OUT "&rst iat= %-23s r1= %10.4f, r2= %10.4f, r3= %10.4f, r4= %10.4f, rk2= %11.7f, rk3= %11.7f, &end #HostTR\n",$atmnums,$vals[$i]-180,$vals[$i],$vals[$i],$vals[$i]+180.0,$akcal,$akcal;}
  #GuestTR
  elsif ($#arr == 1 && $i >= 6 && $i <= 11){printf OUT "&rst iat= %-23s r1= %10.4f, r2= %10.4f, r3= %10.4f, r4= %10.4f, rk2= %11.7f, rk3= %11.7f, &end #GuestTR\n",$atmnums,0.0,$vals[$i],$vals[$i],999.0,$dwt,$dwt;}
  elsif ($#arr == 2 && $i >= 6 && $i <= 11){printf OUT "&rst iat= %-23s r1= %10.4f, r2= %10.4f, r3= %10.4f, r4= %10.4f, rk2= %11.7f, rk3= %11.7f, &end #GuestTR\n",$atmnums,0.0,$vals[$i],$vals[$i],180.0,$awt,$awt;}
  elsif ($#arr == 3 && $i >= 6 && $i <= 11){printf OUT "&rst iat= %-23s r1= %10.4f, r2= %10.4f, r3= %10.4f, r4= %10.4f, rk2= %11.7f, rk3= %11.7f, &end #GuestTR\n",$atmnums,$vals[$i]-180,$vals[$i],$vals[$i],$vals[$i]+180.0,$awt,$awt;}
  #HostJ
  elsif ($#arr == 1 && $i >= 18 && $i <= 20){printf OUT "&rst iat= %-23s r1= %10.4f, r2= %10.4f, r3= %10.4f, r4= %10.4f, rk2= %11.7f, rk3= %11.7f, &end #HostJ\n",$atmnums,0.0,$vals[$i],$vals[$i],999.0,$jwt,$jwt;}
  #GuestJ
  elsif ($#arr == 1 && $i >= 13 && $i <= 15){printf OUT "&rst iat= %-23s r1= %10.4f, r2= %10.4f, r3= %10.4f, r4= %10.4f, rk2= %11.7f, rk3= %11.7f, &end #GuestJ\n",$atmnums,0.0,$vals[$i],$vals[$i],999.0,$lwt,$lwt;}
  #HostDih
  elsif ($#arr == 3 && $i == 17){printf OUT "&rst iat= %-23s r1= %10.4f, r2= %10.4f, r3= %10.4f, r4= %10.4f, rk2= %11.7f, rk3= %11.7f, &end #HostD\n",$atmnums,$RotAngle-180,$RotAngle,$RotAngle,$RotAngle+180.0,$hwt,$hwt;}
  #GuestDih
  elsif ($#arr == 3 && $i == 12){printf OUT "&rst iat= %-23s r1= %10.4f, r2= %10.4f, r3= %10.4f, r4= %10.4f, rk2= %11.7f, rk3= %11.7f, &end #GuestD\n",$atmnums,$vals[$i]-180,$vals[$i],$vals[$i],$vals[$i]+180.0,$cwt,$cwt;}
  elsif ($#arr == 3 && $i == 16){printf OUT "&rst iat= %-23s r1= %10.4f, r2= %10.4f, r3= %10.4f, r4= %10.4f, rk2= %11.7f, rk3= %11.7f, &end #GuestDW\n",$atmnums,$vals[$i-4]-180,$vals[$i-4]-90,$vals[$i-4]+90,$vals[$i-4]+180.0,$wkcal,$wkcal;}
  else {print OUT "UHHH SETUP-Restraints.pl didn't understand something!!!";}
}  

close OUT; # Close disang.rest


if ($Type ne "c" && $Type ne "a") { 
# Write pt-restraints.in for analysis  
open OUT, ">pt-restraints.in";
print OUT "# $H1 $H2 $H3 $G1 $G2 $G3\nnoexitonerror\nparm vac.prmtop\n";
for $i (3..15){printf OUT "trajin md%02.0f.nc\n",$i;}
for $i (0..$#rst){
  if ($rst[$i] =~ /:5000\@O/ || $rst[$i] =~ /:2000\@O/){next;}
  @arr=split " ", $rst[$i];
  if ($#arr == 1 && $i == 6){print OUT "distance d$i $rst[$i] noimage out restraints.dat\nhist d$i,0,30,0.05,* norm out hist.dat\n";}
  elsif ($#arr == 1 && $i >= 7){print OUT "distance d$i $rst[$i] noimage out restraints.dat\n";}
  elsif ($#arr == 2 && $i >= 6){print OUT "angle a$i $rst[$i] out restraints.dat\n";}
  elsif ($#arr == 3 && $i >= 6){print OUT "dihedral a$i $rst[$i] out restraints.dat\n";}
  elsif ($i < 6){next;} #HostTR
  else {print OUT "UHHH SETUP-Restraints.pl is confused!!!";}
}
}
elsif ($Type eq "a") {
open OUT, ">pt-restraints.in";
print OUT "# $H1 $H2 $H3 $G1 $G2 $G3\nnoexitonerror\nparm vac.prmtop\n";
for $i (3..15){printf OUT "trajin md%02.0f.nc\n",$i;}
for $i (0..$#rst){
  if ($rst[$i] =~ /:5000\@O/ || $rst[$i] =~ /:2000\@O/){next;}
  @arr=split " ", $rst[$i];
  if ($#arr == 1 && $i >= 17){print OUT "distance d$i $rst[$i] noimage out restraints.dat\n";}
  elsif ($#arr == 2 && $i >= 17){print OUT "angle a$i $rst[$i] out restraints.dat\n";}
  elsif ($#arr == 3 && $i >= 17){print OUT "dihedral a$i $rst[$i] out restraints.dat\n";}
  elsif ($i < 17){next;} #HostTR
  else {print OUT "UHHH SETUP-Restraints.pl is confused!!!";}
}
}
else {
open OUT, ">pt-restraints.in";
print OUT "# $H1 $H2 $H3 $G1 $G2 $G3\nnoexitonerror\nparm vac.prmtop\n";
for $i (3..15){printf OUT "trajin md%02.0f.nc\n",$i;}
for $i (0..$#rst){
  if ($rst[$i] =~ /:5000\@O/ || $rst[$i] =~ /:2000\@O/){next;}
  @arr=split " ", $rst[$i];
  if ($#arr == 1 && $i >= 12){print OUT "distance d$i $rst[$i] noimage out restraints.dat\n";}
  elsif ($#arr == 2 && $i >= 12){print OUT "angle a$i $rst[$i] out restraints.dat\n";}
  elsif ($#arr == 3 && $i >= 12){print OUT "dihedral a$i $rst[$i] out restraints.dat\n";}
  elsif ($i < 12){next;} #HostTR
  else {print OUT "UHHH SETUP-Restraints.pl is confused!!!";}
}
}
#}
#}
