#!/usr/bin/perl

### Check Args
if ($#ARGV < 0) {
    print "#### SETUP-Solvate.pl ##########################################################################\n\n";
    print "Usage: perl SETUP-Solvate.pl LeapFile TransDist\n\n";
    exit;
}


### Translation of the ligand
$LeapFile=$ARGV[0];
$TransDist=$ARGV[1];
$LoadInfo=`cat $ARGV[0] | sed 's/quit//g'`;
#$NumAtoms=36000;
$NumAdded=0;
$Factor=1;

### Buffer

$Buff=30.0;
if ($TransDist >= 3.20){
$distance=($TransDist - 3.20);
$Buff -= 0.5*$distance;
}

foreach (@ARGV){
  if ($_ eq '--example'){
    print "####### Below is an example Solvate.in input file #######\n\n";
    print <<EOF;
source leaprc.gaff
source leaprc.protein.ff14SB
loadoff atomic_ions.lib
loadOff solvents.lib
HOH = SPC
WAT = SPC
loadAmberParams frcmod.spce
loadamberparams frcmod.ionsjc_spce
loadamberparams mmm.frcmod
MMM = loadmol2 mmm.mol2
loadamberparams dum.frcmod
DUM = loadmol2 dum.mol2
model = loadpdb build.pdb

check model
savepdb model vac.pdb
saveamberparm model vac.prmtop vac.crds

solvatebox model SPCBOX {11.0 11.0 26.0}
addionsrand model Na+ 32
addionsrand model Cl-  0

#savepdb model full.pdb
#saveamberparm model full.prmtop full.crds

desc model

quit
EOF
    exit;
  }
}


open BUILDIT, ">Solvate.in";
print BUILDIT <<EOF;
###########################################################
### Leap Input File     ($LeapFile)

$LoadInfo
##################

solvatebox model SPCBOX {11.0 11.0 $Buff}
addionsrand model Na+ 32
addionsrand model Cl-  0

#savepdb model full.pdb
#saveamberparm model full.prmtop full.crds

desc model

quit
EOF
close BUILDIT;

## Check AMBERHOME
$AMBERHOME=`echo \$AMBERHOME`;

chomp($AMBERHOME);
if ($AMBERHOME eq ""){die "\n  AMBERHOME is not set\n";}

$HMR='Yes';


system "perl -p -i -e 's/#savepdb/savepdb/g' Solvate.in";
system "perl -p -i -e 's/#saveamberparm/saveamberparm/g' Solvate.in";
system "$AMBERHOME/bin/tleap -s -f Solvate.in > Solvate.log";

$NumVac=`tail -3 vac.pdb | head -1 | awk '{print \$2}'`; chomp($NumVac);
$NumAtoms=$NumVac+33067;

$Temp=$NumAtoms;
$Num=`grep "$Temp  H2" full.pdb | awk '{print \$2}'`; chomp($Num);
$Temp=$NumAtoms+1;
$Num=`grep "$Temp  H2" full.pdb | awk '{print \$2}'`; chomp($Num);
if ($Num ne ""){$NumAtoms=$Num};
$Temp=$NumAtoms+2;
$Num=`grep "$Temp  H2" full.pdb | awk '{print \$2}'`; chomp($Num);
if ($Num ne ""){$NumAtoms=$Num};
print "$NumAtoms\n";


$NumAdded=`head -2 full.crds | tail -1`; chomp($NumAdded);
$Diff= $NumAdded-$NumAtoms-30;
print "$NumAdded\n";
print "$Diff\n";

$Buff -= $Diff/751.75;

$NumMax=$NumAtoms+60;

while ($NumAdded < $NumAtoms || $NumAdded > $NumMax) {

open BUILDIT, ">Solvate.in";
print BUILDIT <<EOF;
###########################################################
### Leap Input File     ($LeapFile)

$LoadInfo
##################

solvatebox model SPCBOX {11.0 11.0 $Buff}
addionsrand model Na+ 32
addionsrand model Cl-  0

#savepdb model full.pdb
#saveamberparm model full.prmtop full.crds

desc model

quit
EOF
close BUILDIT;

## Check AMBERHOME
$AMBERHOME=`echo \$AMBERHOME`;
chomp($AMBERHOME);
if ($AMBERHOME eq ""){die "\n  AMBERHOME is not set\n";}

$HMR='Yes';


system "perl -p -i -e 's/#savepdb/savepdb/g' Solvate.in";
system "perl -p -i -e 's/#saveamberparm/saveamberparm/g' Solvate.in";
system "$AMBERHOME/bin/tleap -s -f Solvate.in > Solvate.log";

$NumAdded=`head -2 full.crds | tail -1`; chomp($NumAdded);
$Diff= $NumAdded-$NumAtoms-30;
$Buff -= $Factor*$Diff/751.75;
print "$NumAdded\n";
print "$Diff\n";
$Factor = $Factor*0.8;
}



open BUILDIT, ">Adjust.in";
print BUILDIT <<EOF;
###########################################################
#### Leap Input File     ($LeapFile)

$LoadInfo
##################

solvatebox model SPCBOX 0 10
#addionsrand model Na+ 32
#addionsrand model Cl-  0

savepdb model full.pdb
saveamberparm model full.prmtop full.crds

desc model

quit
EOF
close BUILDIT;

$Linecut=`awk '/$NumAtoms/{print NR}' full.pdb`; chomp($Linecut);
print "$Linecut";
system "head -$Linecut full.pdb > full-adj.pdb";
system "tail -2 full.pdb >> full-adj.pdb";

$HMR='Yes';


system "perl -p -i -e 's/loadpdb build.pdb/loadpdb full-adj.pdb/g' Adjust.in";
system "perl -p -i -e 's/savepdb model vac.pdb/#savepdb model vac.pdb/g' Adjust.in";
system "perl -p -i -e 's/saveamberparm model vac.prmtop vac.crds/#saveamberparm model vac.prmtop vac.crds/g' Adjust.in";
#system "perl -p -i -e 's/#savepdb/savepdb/g' Adjust.in";
#system "perl -p -i -e 's/#saveamberparm/saveamberparm/g' Adjust.in";
system "$AMBERHOME/bin/tleap -s -f Adjust.in > Adjust.log";




if ($HMR eq "Yes"){
  print "\n###########################################################\n### HMR Requested.  HMR prmtop name is \"full.hmr.prmtop\"\n";
  open OUT, ">parmed-hmr.in";
  print OUT <<EOF;
parm full.prmtop
HMassRepartition
parmout full.hmr.prmtop
EOF
  close OUT;
  system "parmed -O -n -i parmed-hmr.in > parmed-hmr.log";
} 


$Errors=`egrep 'ERROR|WARNING' Solvate.log`;
chomp($Errors);
print <<EOF;

###########################################################
### Errors/Warnings in Logfile.
$Errors
EOF


print <<EOF;

###########################################################
### Always inspect Solvate.in, Solvate.log, full.prmtop,
### full.crds, and full.pdb for errors.  
### --Niel
###########################################################

EOF
