
### BE SURE THAT THE ENERGY CALCS MATCH THE DISANG FILE ORDER

import numpy as np # numerical array library
import subprocess as sp
import sys,os
from akima import interpolate
import pymbar # multistate Bennett acceptance ratio
import timeseries # timeseries analysis

### Arguments
aur = sys.argv[1] # a or u or r or d
temp = float(sys.argv[2]) # temp
kB = 1.381e-23 * 6.022e23 / (4.184 * 1000.0) # Boltzmann constant in kJ/mol/K
beta = 1/(kB * temp) # beta
N_max = 2000000 # Max frames for any simulation window, you should check this if you did some long runs

sys.stdout = open('mbar-'+aur+'.log', 'w')
### Determine Number of umbrellas
K = 0
if aur == 'd':
  filename = './'+aur+'%03.0f/restraints.dat' % K
  while os.path.isfile(filename):
    K = K+1
    filename = './'+aur+'%03.0f/restraints.dat' % K
else:
  filename = './'+aur+'%02.0f/restraints.dat' % K
  while os.path.isfile(filename):
    K = K+1
    filename = './'+aur+'%02.0f/restraints.dat' % K


### Determine Number of restraints
infile = open('./c00/disang.rest', 'r')
disang = infile.readlines()
infile.close()
R = 0
for line in disang:
  cols = line.split()
  if len(cols) != 0 and (cols[-1] == "#GuestJ" or cols[-1] == "#GuestD"):
    R += 1

print  "K= %5.0f  R= %5.0f" % ( K, R ) 

### Functions
def factors(n):   # Return list of integer factors
  facs=[]
  sqrt=int(round(np.sqrt(n)+0.5))
  i = 1
  while i <= sqrt:
    if n % i == 0:
      facs.append(i)
      j=n/i
      if j != i:
        facs.append(j)
    i += 1
  return sorted(facs, key=int)

def nearestmax(n):  # Nearest number that is 100 less than input and divisible by two
  int=[]
  numfac=[]
  maxfac=0
  if n % 2 == 0:
    low=n-100
    high=n
  else:
    low=n-101
    high=n-1
  if low < 0:
    low=0
  for i in range(low,high+2,2):
    numfac = len(factors(i))
    if numfac >= maxfac:
      maxfac = numfac
      mostfac = i
  return mostfac

def seom(N,arr):
  Facs=[]
  SEOM=[]
  Facs=factors(N)
  Bn=np.zeros([len(Facs)], np.int32) # Number of blocks for a given block size
  Bmean=np.zeros([len(Facs),Facs[-1]], np.float64) # Means for each block of each block size
  SEOM=np.zeros([len(Facs)-2],np.float64)
  for i in range(len(Facs)-2):   # Run over all block sizes except the final two: two blocks, one block.
    for j in range(Facs[-i-1]):  # Run over all blocks in the data for a specific size.
      Bmean[i,j]=np.mean(arr[j*Facs[i]:(j+1)*Facs[i]])
    Bn[i]=j+1
    SEOM[i]=np.std(Bmean[i,0:Bn[i]],ddof=0)/np.sqrt(Bn[i]-1)
  return np.max(SEOM)
  #hist,edges = np.histogram(SEOM,range=[np.min(SEOM),np.max(SEOM)],bins=50,)  ### edges = len(hist)+1
  #return np.max(SEOM),edges[np.argmax(hist)+1]  # Assume the blocking plateau corresponds to the max histogram count!

### Calculate Statistical Inefficiency (g)
def calcg(data):
  sum = 0
  randnum = ("%05.0f" % (int(100000*np.random.random())))
  datafn = '/dev/shm/series.'+randnum+'.dat'
  acffn = '/dev/shm/acf.'+randnum+'.dat'
  cppfn = '/dev/shm/pt-acf.'+randnum+'.in'
  np.savetxt(datafn,data)
  cpptin = open(cppfn, 'w')
  cpptin.write("readdata "+datafn+" name "+randnum+"\nautocorr "+randnum+" out "+acffn+" noheader\n")
  cpptin.close()

  FNULL = open(os.devnull, 'w')
  sp.call(['./cpptraj','-i',cppfn], stdout=FNULL, stderr=sp.STDOUT)

  with open(acffn, 'r') as acf:
    for line in acf:
      col = line.split()
      t = float(col[0]) - 1.0
  T = t

  with open(acffn, 'r') as acf:
    for line in acf:
      col = line.split()
      t = float(col[0]) - 1.0
      v = float(col[1])
      if t == 0:
        continue
      if v < 0.0:
        break
      sum += ( 1 - (t/T) )*(v)

  sp.call(['rm',datafn,acffn,cppfn])

  return 1+(2*sum)


### Allocate storage for simulation data
N = np.zeros([K], np.int32)                       # N_k[k] is the number of snapshots to be used from umbrella simulation k
Neff = np.zeros([K], np.int32)
Nind = np.zeros([K], np.int32)
Nprg = np.zeros([K], np.int32)
rty = ['d']*R                                     # restraint type (distance or angle)
rfc = np.zeros([K,R], np.float64)                 # restraint force constant
fcmax = np.zeros([R], np.float64)                 # full force constant value used during umbrella portion of work 
req = np.zeros([K,R], np.float64)                 # restraint target value
val = np.zeros([N_max,K,R], np.float64)           # value of the restrained variable at each frame n
g = np.zeros([K], np.float64)

### Tmp type arrays for energy and spline fitting/integration
u=np.zeros([N_max], np.float64)
x=np.zeros([K],np.float64)
y=np.zeros([K],np.float64)
m=np.zeros([K],np.float64)
s=np.zeros([K],np.float64)

print "Done with array setup\n"

### Read the simulation data
for k in range(K):
  print ".",
  # Read Equilibrium Value and Force Constant
  if aur == 'd':
    filename = './'+aur+'%03.0f/disang.rest' % k
  else:
    filename = './'+aur+'%02.0f/disang.rest' % k
  infile = open(filename, 'r')
  disang = infile.readlines()
  infile.close()
  print ".",
  r = 0
  for line in disang:
    cols = line.split()
    if len(cols) != 0 and (cols[-1] == "#GuestJ" or cols[-1] == "#GuestD"):
      natms = len(cols[2].split(','))-1
      req[k,r] = float(cols[6].replace(",", ""))
      if natms == 2:
        rty[r] = 'd'
        rfc[k,r] = float(cols[12].replace(",", ""))
      elif natms == 3:
        rty[r] = 'a'
        rfc[k,r] = float(cols[12].replace(",", ""))*(np.pi/180.0)*(np.pi/180.0)  ### Convert to degrees
      elif natms == 4:
        rty[r] = 't'
        rfc[k,r] = float(cols[12].replace(",", ""))*(np.pi/180.0)*(np.pi/180.0)  ### Convert to degrees
      else:
        sys.exit("not sure about restraint type!")
      r += 1

  print ".",
  # Read in Values for restrained variables for each simulation
  if aur == 'd':
    filename = './'+aur+'%03.0f/restraints.dat' % k
  else:
    filename = './'+aur+'%02.0f/restraints.dat' % k
  infile = open(filename, 'r')
  restdat = infile.readlines()     # slice off first 20 lines  readlines()[20:]
  infile.close()
  print ".",
  # Parse Data
  n = 0
  for line in restdat:
    if line[0] != '#' and line[0] != '@' and n < N_max:
      cols = line.split()
      for r in range(R): 
        if rty[r] == 't': # Do phase corrections
          tmp = float(cols[r+1])
          if tmp < req[k,r]-180.0:
            val[n,k,r] = tmp + 360
          elif tmp > req[k,r]+180.0:
            val[n,k,r] = tmp - 360
          else:
            val[n,k,r] = tmp
        else:
          val[n,k,r] = float(cols[r+1])
      n += 1

  print ".",
  # Set N For Each Window
  #N[k] = nearestmax(n)
  N[k] = n
  if n > N_max:
    sys.exit("I'ma die now 'cause the number of data points exceeds N_max hardlimit")

  # Calculate Reduced Potential 
  if aur == 'a' or aur == 'd' or aur == 'l' or aur == 'c': ### Attach Restraints
    if rfc[k,0] == 0:
      tmp=np.ones([R],np.float64)*0.001
      u[0:N[k]] = np.sum(beta*tmp[0:R]*((val[0:N[k],k,0:R]-req[k,0:R])**2), axis=1)
    else:
      u[0:N[k]] = np.sum(beta*rfc[k,0:R]*((val[0:N[k],k,0:R]-req[k,0:R])**2), axis=1)
  elif aur == 'r' or aur == 'm': ### Release just jacks!
    if rfc[k,0] == 0:
      tmp=np.ones([R],np.float64)*0.001
      u[0:N[k]] = np.sum(beta*tmp[0:R]*((val[0:N[k],k,0:R]-req[k,0:R])**2), axis=1) # If Independent release phase
      #u[0:N[k]] = np.sum(beta*tmp[3:R]*((val[0:N[k],k,3:R]-req[k,3:R])**2), axis=1) # If combined with attach and umbrella phases
    else:
      #u[0:N[k]] = np.sum(beta*rfc[k,0:R-3]*((val[0:N[k],k,0:R-3]-req[k,0:R-3])**2), axis=1) # OLD STYLE If combined with attach and umbrella phases
      u[0:N[k]] = np.sum(beta*rfc[k,0:R]*((val[0:N[k],k,0:R]-req[k,0:R])**2), axis=1) # If Independent release phase
      #u[0:N[k]] = np.sum(beta*rfc[k,3:R]*((val[0:N[k],k,3:R]-req[k,3:R])**2), axis=1) # If combined with attach and umbrella phases
  else: ### Umbrella/Translation
    #u[0:N[k]] = (beta*rfc[k,-3]*((val[0:N[k],k,-3]-req[k,-3])**2)) # OLD STYLE disang file
    u[0:N[k]] = (beta*rfc[k,0]*((val[0:N[k],k,0]-req[k,0])**2))


  print ".",
  g[k] = calcg(u[0:N[k]])
  subs = timeseries.subsampleCorrelatedData(np.zeros([N[k]]),g=g[k])
  Nind[k] = len(subs)
  if Nind[k] > 100000:
    Neff[k] = 100000
  else:
    Neff[k] = Nind[k]
  
  print  "Processed Window %5.0f.  N= %12.0f.  g= %10.3f   Nind= %12.0f   Neff= %12.0f" % ( k, N[k], g[k], Nind[k], Neff[k] ) 

print  "Max Neff= %.0f" % ( np.max(Neff) ) 
Upot = np.zeros([K,K,np.max(Neff)], np.float64)


# Calculate Restraint Energy
for k in range(K):
  subs = timeseries.subsampleCorrelatedData(np.zeros([N[k]]),g=g[k])
  for l in range(K):
    if aur == 'a' or aur == 'd' or aur == 'l' or aur == 'c': ### Attach Restraints
      Upot[k,l,0:Neff[k]] = np.sum(beta*rfc[l,0:R]*((val[subs[0:Neff[k]],k,0:R]-req[l,0:R])**2), axis=1)
    elif aur == 'r' or aur == 'm': ### Release just jacks!
      #Upot[k,l,0:Neff[k]] = np.sum(beta*rfc[l,0:R-3]*((val[subs[0:Neff[k]],k,0:R-3]-req[l,0:R-3])**2), axis=1) # OLD STYLE If combined with attach and umbrella phases
      Upot[k,l,0:Neff[k]] = np.sum(beta*rfc[l,0:R]*((val[subs[0:Neff[k]],k,0:R]-req[l,0:R])**2), axis=1) # If Independent release phase
      #Upot[k,l,0:Neff[k]] = np.sum(beta*rfc[l,3:R]*((val[subs[0:Neff[k]],k,3:R]-req[l,3:R])**2), axis=1) # If combined with attach and umbrella phases
    else: ### Umbrella/Translation
      #Upot[k,l,0:Neff[k]] = (beta*rfc[l,-3]*((val[subs[0:Neff[k]],k,-3]-req[l,-3])**2)) # OLD STYLE disang file
      Upot[k,l,0:Neff[k]] = (beta*rfc[l,0]*((val[subs[0:Neff[k]],k,0]-req[l,0])**2))


val=[]

prg = [100]
#prg = [100]
for p in range(len(prg)):

  Nprg = Neff*prg[p]/100 ## Test integers out only 
  print  "Running MBAR on %.0f percent of the data ... " % ( prg[p] ) 
  mbar = pymbar.MBAR(Upot, Nprg, verbose = True, method = 'adaptive', initialize = 'BAR')

  print  "Calculate Free Energy Differences Between States" 
  [Deltaf, dDeltaf] = mbar.getFreeEnergyDifferences()

  min = np.argmin(Deltaf[0])

  # Write to file
  print  "Free Energy Differences (in units of kcal/mol)" 
  print  "%9s %8s %8s %12s %12s" % ('bin', 'f', 'df', 'deq', 'dfc') 
  datfile = open('mbar-'+aur+'.%03.0f.dat' % prg[p], 'w')
  for k in range(K):
    if aur == 'a' or aur == 'r' or aur == 'l' or aur == 'm' or aur == 'c':
      print "%10.5f %10.5f %10.5f %12.7f %12.7f" % ( rfc[k,0]/rfc[-1,0], Deltaf[0,k]/beta, dDeltaf[0,k]/beta, req[k,0], rfc[k,0] )
      datfile.write ( "%10.5f %10.5f %10.5f %12.7f %12.7f\n" % ( rfc[k,0]/rfc[-1,0], Deltaf[0,k]/beta, dDeltaf[0,k]/beta, req[k,0], rfc[k,0] ) )
    elif aur == 'd':
      print "%9.0f %10.5f %10.5f %12.7f %12.7f" % ( k, Deltaf[0,k]/beta, dDeltaf[0,k]/beta, req[k,0], rfc[k,0]/rfc[-1,0] )
      datfile.write ( "%9.0f %10.5f %10.5f %12.7f %12.7f\n" % ( k, Deltaf[0,k]/beta, dDeltaf[0,k]/beta, req[k,0], rfc[k,0]/rfc[-1,0] ) )
    else: # 'u'
      print "%10.5f %10.5f %10.5f %12.7f %12.7f" % ( req[k,0], Deltaf[0,k]/beta, dDeltaf[0,k]/beta, req[k,0], rfc[k,0] )
      datfile.write ( "%10.5f %10.5f %10.5f %12.7f %12.7f\n" % ( req[k,0], Deltaf[0,k]/beta, dDeltaf[0,k]/beta, req[k,0], rfc[k,0] ) )
  datfile.close()
  print "\n\n"

